//
//  CXDataStore.m
//  Chroniton
//
//  Created by Logan Collins on 4/20/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXDataStore.h"
#import "CXDataStore_Private.h"


@implementation CXDataStore {
	NSMutableDictionary *_values;
}

+ (CXDataStore *)dataStore {
	return [[self alloc] init];
}

+ (CXDataStore *)dataStoreWithContentsOfURL:(NSURL *)aURL error:(NSError **)outError {
	return [(CXDataStore *)[self alloc] initWithContentsOfURL:aURL error:outError];
}

- (id)init {
	self = [super init];
	if (self) {
		_values = [NSMutableDictionary dictionary];
	}
	return self;
}

- (id)initWithContentsOfURL:(NSURL *)aURL error:(NSError **)outError {
	self = [self init];
	if (self) {
		NSError *error = nil;
		
		NSData *data = [NSData dataWithContentsOfURL:aURL options:0 error:&error];
		if (data != nil) {
			NSDictionary *values = [NSPropertyListSerialization propertyListWithData:data options:0 format:NULL error:&error];
			if (values != nil) {
				[_values setDictionary:values];
			}
		}
		
		if (outError != nil) {
			*outError = error;
		}
		
		if (error != nil) {
			return nil;
		}
	}
	return self;
}

- (BOOL)writeToURL:(NSURL *)aURL error:(NSError **)outError {
	NSError *error = nil;
	
	NSData *data = [NSPropertyListSerialization dataWithPropertyList:_values format:NSPropertyListXMLFormat_v1_0 options:0 error:&error];
	if (data != nil) {
		[data writeToURL:aURL options:NSDataWritingAtomic error:&error];
	}
	
	if (outError != NULL) {
		*outError = error;
	}
	
	return (error == nil);
}


#pragma mark -
#pragma mark Value accessors

- (id)valueForUndefinedKey:(NSString *)key {
	id value = nil;
	@synchronized(self) {
		value = [_values objectForKey:key];
	}
	return value;
}

- (void)setValue:(id)value forUndefinedKey:(NSString *)key {
	@synchronized(self) {
		[_values setObject:value forKey:key];
	}
}

- (void)removeObjectForKey:(NSString *)key {
	@synchronized(self) {
		[_values removeObjectForKey:key];
	}
}

- (NSDictionary *)dictionaryRepresentation {
	NSDictionary *dictionary = nil;
	@synchronized(self) {
		dictionary = [_values copy];
	}
	return dictionary;
}

- (void)setDictionaryRepresentation:(NSDictionary *)dictionary {
	@synchronized(self) {
		[_values setDictionary:dictionary];
	}
}


#pragma mark -
#pragma mark Type accessors

- (id)objectForKey:(NSString *)key {
	return [self valueForKey:key];
}

- (void)setObject:(id)value forKey:(NSString *)key {
	[self setValue:value forKey:key];
}

- (NSArray *)arrayForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSArray class]]) {
		value = nil;
	}
	return value;
}

- (void)setArray:(NSArray *)array forKey:(NSString *)key {
	[self setValue:[array copy] forKey:key];
}

- (NSData *)dataForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSData class]]) {
		value = nil;
	}
	return value;
}

- (void)setData:(NSData *)value forKey:(NSString *)key {
	[self setValue:[value copy] forKey:key];
}

- (NSDictionary *)dictionaryForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSDictionary class]]) {
		value = nil;
	}
	return value;
}

- (void)setDictionary:(NSDictionary *)dictionary forKey:(NSString *)key {
	[self setValue:[dictionary copy] forKey:key];
}

- (NSString *)stringForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSString class]]) {
		value = nil;
	}
	return value;
}

- (void)setString:(NSString *)value forKey:(NSString *)key {
	[self setValue:[value copy] forKey:key];
}

- (BOOL)boolForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSNumber class]]) {
		value = nil;
	}
	return [value boolValue];
}

- (void)setBool:(BOOL)value forKey:(NSString *)key {
	[self setValue:[NSNumber numberWithBool:value] forKey:key];
}

- (NSInteger)integerForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSNumber class]]) {
		value = nil;
	}
	return [value integerValue];
}

- (void)setInteger:(NSInteger)value forKey:(NSString *)key {
	[self setValue:[NSNumber numberWithInteger:value] forKey:key];
}

- (float)floatForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSNumber class]]) {
		value = nil;
	}
	return [value floatValue];
}

- (void)setFloat:(float)value forKey:(NSString *)key {
	[self setValue:[NSNumber numberWithFloat:value] forKey:key];
}

- (double)doubleForKey:(NSString *)key {
	id value = [self valueForKey:key];
	if (value != nil && ![value isKindOfClass:[NSNumber class]]) {
		value = nil;
	}
	return [value doubleValue];
}

- (void)setDouble:(double)value forKey:(NSString *)key {
	[self setValue:[NSNumber numberWithDouble:value] forKey:key];
}

@end
