//
//  CXDirector.h
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <QuartzCore/QuartzCore.h>
#import <Chroniton/ChronitonDefines.h>
#import <Chroniton/CXResponder.h>


@class CXScene, CXTimer;
@protocol CXDirectorDelegate;


/*!
 * @class CXDirector
 * @abstract Coordinates scenes and timing
 */
@interface CXDirector : NSObject <CXResponder>

/*!
 * @method sharedDirector
 * @abstract The shared director object
 * 
 * @result A CXDirector object
 */
+ (CXDirector *)sharedDirector;


/*!
 * @property delegate
 * @abstract The director delegate object
 * 
 * @result An object conforming to the CXDirectorDelegate protocol
 */
@property (weak, readonly) id <CXDirectorDelegate> delegate;


/*!
 * @property baseLayer
 * @abstract The base layer within scenes are attached
 * 
 * @result A CALayer object
 */
@property (strong, readonly) CALayer *baseLayer;


/*!
 * @property paused
 * @abstract Whether scenes and timers are currently paused
 * 
 * @result A BOOL value
 */
@property (readonly, getter=isPaused) BOOL paused;

/*!
 * @method pause
 * @abstract Pauses the scenes and timers
 */
- (void)pause;

/*!
 * @method resume
 * @abstract Resumes the scenes and timers
 */
- (void)resume;

/*!
 * @method scheduleTimerWithInterval:repeats:block:
 * @abstract Creates a directed timer
 * 
 * @param interval
 * The interval before the timer fires
 * 
 * @param repeats
 * Whether the timer is automatically rescheduled upon firing
 * 
 * @param block
 * The block to invoke upon firing
 * 
 * @discussion
 * The timer will fire when the interval has elapsed, calling the block.
 * If the director is paused, the timer will pause as well, and resume
 * when the director resumes.
 * 
 * The object returned from this method can be passed to -cancelTimer:.
 * 
 * @result A CXTimer object
 */
- (CXTimer *)scheduleTimerWithInterval:(NSTimeInterval)interval repeats:(BOOL)repeats block:(void (^)(void))block;

/*!
 * @method invalidateTimer:
 * @abstract Invalidates a scheduled timer
 * 
 * @param timer
 * A timer returned from scheduleTimerWithInterval:repeats:block:
 * 
 * @discussion
 * If the timer is non-repeating and has already fired, or has already
 * been invalidated, this method has no effect.
 */
- (void)invalidateTimer:(CXTimer *)timer;


/*!
 * @property scenes
 * @abstract All scenes currently in the scene stack
 * 
 * @result An NSArray of CXScene objects
 */
@property (copy, readonly) NSArray *scenes;

/*!
 * @property currentScene
 * @abstract The current scene
 * 
 * @discussion
 * Setting the current scene is useful in situations where there
 * is no need to transition back to the previous scene, such as
 * advancements in storyline.
 * 
 * If the scene is already in the scene stack, this method
 * raises an NSInvalidArgument exception.
 * 
 * @result A CXScene object
 */
@property (strong) CXScene *currentScene;

/*!
 * @method pushScene:
 * @abstract Pushes a scene onto the scene stack
 * 
 * @param scene
 * The scene to push
 * 
 * @discussion
 * Pushing a new scene onto the stack is useful in situations
 * where there is a need to transition back to the previous scene,
 * such as a minigame, puzzle or battle.
 * 
 * If the scene has already been pushed onto the stack, this method
 * raises an NSInvalidArgument exception.
 */
- (void)pushScene:(CXScene *)scene;

/*!
 * @method popScene
 * @abstract Pops the current scene off of the scene stack
 */
- (void)popScene;

@end


@protocol CXDirectorDelegate <NSObject>

@optional

/*!
 * @method directorDidPause:
 * @abstract The director has paused
 * 
 * @param director
 * The director object
 */
- (void)directorDidPause:(CXDirector *)director;

/*!
 * @method directorDidResume:
 * @abstract The director has resumed
 * 
 * @param director
 * The director object
 */
- (void)directorDidResume:(CXDirector *)director;


/*!
 * @method director:didPushScene:
 * @abstract A scene has been pushed onto the scene stack
 * 
 * @param director
 * The director object
 * 
 * @param scene
 * The scene that was pushed
 */
- (void)director:(CXDirector *)director didPushScene:(CXScene *)scene;

/*!
 * @method director:didPopScene:
 * @abstract A scene has been popped off of the scene stack
 * 
 * @param director
 * The director object
 * 
 * @param scene
 * The scene that was popped
 */
- (void)director:(CXDirector *)director willPopScene:(CXScene *)scene;

@end


/*!
 * @constant CXDirectorDidPauseNotification
 * @abstract The director has paused
 */
CHRONITON_EXTERN NSString * const CXDirectorDidPauseNotification;

/*!
 * @constant CXDirectorDidResumeNotification
 * @abstract The director has resumed
 */
CHRONITON_EXTERN NSString * const CXDirectorDidResumeNotification;

