//
//  CXDirector.m
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXDirector.h"
#import "CXDirector_Private.h"

#import "CXTimer_Private.h"
#import "CXScene_Private.h"
#import "CXLayer_Private.h"
#import "CXEvent_Private.h"

#import "CXMapTable.h"


NSString * const CXDirectorDidPauseNotification = @"CXDirectorDidPauseNotification";
NSString * const CXDirectorDidResumeNotification = @"CXDirectorDidResumeNotification";


@interface CXDirector ()

- (void)invalidateAllTimers;

@end


@implementation CXDirector {
	NSMutableArray *_timers;
	CXMapTable *_runLoopTimersToTimers;
	NSMutableOrderedSet *_scenes;
    id _responderTarget;
}

@synthesize delegate=_delegate;

@synthesize baseLayer=_baseLayer;
@synthesize paused=_paused;

static CXDirector *sharedDirector = nil;

+ (CXDirector *)sharedDirector {
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedDirector = [[self alloc] init];
	});
	return sharedDirector;
}

+ (id)allocWithZone:(NSZone *)zone {
    // There can only be one.
    if (sharedDirector != nil) {
        return nil;
    }
    return [super allocWithZone:zone];
}

- (id)init {
	self = [super init];
	if (self) {
		_baseLayer = [CALayer layer];
		
		_timers = [NSMutableArray array];
		_runLoopTimersToTimers = [CXMapTable mapTableWithStrongToStrongObjects];
		
		_scenes = [NSMutableOrderedSet orderedSet];
	}
	return self;
}

- (void)dealloc {
	[self invalidateAllTimers];
}


#pragma mark -
#pragma mark Responder

- (id <CXResponder>)nextResponder {
    return nil;
}

- (void)handle_keyDown:(CXEvent *)theEvent {
    if ([[self currentScene] respondsToSelector:@selector(keyDown:)]) {
        [[self currentScene] keyDown:theEvent];
    }
}

- (void)handle_keyUp:(CXEvent *)theEvent {
    if ([[self currentScene] respondsToSelector:@selector(keyUp:)]) {
        [[self currentScene] keyUp:theEvent];
    }
}

- (void)handle_mouseMoved:(CXEvent *)theEvent {
    
}

- (void)handle_mouseDown:(CXEvent *)theEvent {
    CGPoint thePoint = [[self currentScene] convertPointFromBase:[theEvent location]];
    id target = [[[self currentScene] layer] hitTest:thePoint];
    if (target == nil) {
        target = [self currentScene];
    }
    
    _responderTarget = target;
    
    if ([_responderTarget respondsToSelector:@selector(mouseDown:)]) {
        [_responderTarget mouseDown:theEvent];
    }
}

- (void)handle_mouseDragged:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(mouseDragged:)]) {
        [_responderTarget mouseDragged:theEvent];
    }
}

- (void)handle_mouseUp:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(mouseUp:)]) {
        [_responderTarget mouseUp:theEvent];
    }
    _responderTarget = nil;
}

- (void)handle_rightMouseDown:(CXEvent *)theEvent {
    CGPoint thePoint = [[self currentScene] convertPointFromBase:[theEvent location]];
    id target = [[[self currentScene] layer] hitTest:thePoint];
    if (target == nil) {
        target = [self currentScene];
    }
    
    _responderTarget = target;
    
    if ([_responderTarget respondsToSelector:@selector(rightMouseDown:)]) {
        [_responderTarget rightMouseDown:theEvent];
    }
}

- (void)handle_rightMouseDragged:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(rightMouseDragged:)]) {
        [_responderTarget rightMouseDragged:theEvent];
    }
}

- (void)handle_rightMouseUp:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(rightMouseUp:)]) {
        [_responderTarget rightMouseUp:theEvent];
    }
    _responderTarget = nil;
}

- (void)handle_touchesBegan:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    CXTouch *touch = [touches anyObject];
    CGPoint thePoint = [[self currentScene] convertPointFromBase:[touch location]];
    id target = [[[self currentScene] layer] hitTest:thePoint];
    if (target == nil) {
        target = [self currentScene];
    }
    
    _responderTarget = target;
    
    if ([_responderTarget respondsToSelector:@selector(touchesBegan:withEvent:)]) {
        [_responderTarget touchesBegan:touches withEvent:theEvent];
    }
}

- (void)handle_touchesMoved:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(touchesMoved:withEvent:)]) {
        [_responderTarget touchesMoved:touches withEvent:theEvent];
    }
}

- (void)handle_touchesEnded:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(touchesEnded:withEvent:)]) {
        [_responderTarget touchesEnded:touches withEvent:theEvent];
    }
    _responderTarget = nil;
}

- (void)handle_touchesCancelled:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    if ([_responderTarget respondsToSelector:@selector(touchesCancelled:withEvent:)]) {
        [_responderTarget touchesCancelled:touches withEvent:theEvent];
    }
    _responderTarget = nil;
}

- (void)handle_motionBegan:(CXEvent *)theEvent {
    
}

- (void)handle_motionEnded:(CXEvent *)theEvent {
    
}

- (void)handle_motionCancelled:(CXEvent *)theEvent {
    
}


#pragma mark -
#pragma mark Timing

- (void)pause {
	if (_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	
	_paused = YES;
	
	// Deschedule timers
	for (CXTimer *timer in _timers) {
		NSTimer *runLoopTimer = [timer runLoopTimer];
		NSDate *fireDate = [runLoopTimer fireDate];
		
		NSDate *date = [NSDate date];
		NSTimeInterval remainingTimeInterval = [fireDate timeIntervalSinceDate:date];
		
		timer.nextInterval = remainingTimeInterval;
		
		[runLoopTimer invalidate];
		[_runLoopTimersToTimers removeObjectForKey:runLoopTimer];
	}
	
	[self didChangeValueForKey:@"paused"];
	
	if ([[self delegate] respondsToSelector:@selector(directorDidPause:)]) {
		[[self delegate] directorDidPause:self];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:CXDirectorDidPauseNotification object:self];
}

- (void)resume {
	if (!_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	
	_paused = NO;
	
	// Reschedule timers
	for (CXTimer *timer in _timers) {
		NSTimeInterval nextInterval = [timer nextInterval];
		NSTimer *runLoopTimer = [NSTimer scheduledTimerWithTimeInterval:nextInterval target:self selector:@selector(executeTimer:) userInfo:nil repeats:timer.repeats];
		
		timer.runLoopTimer = runLoopTimer;
		
		[_runLoopTimersToTimers setObject:timer forKey:runLoopTimer];
	}
	
	[self didChangeValueForKey:@"paused"];
	
	if ([[self delegate] respondsToSelector:@selector(directorDidResume:)]) {
		[[self delegate] directorDidResume:self];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:CXDirectorDidResumeNotification object:self];
}

- (void)executeTimer:(NSTimer *)runLoopTimer {
	CXTimer *timer = [_runLoopTimersToTimers objectForKey:runLoopTimer];
	void (^block)(void) = timer.block;
	if (block != nil) {
		block();
	}
	
	// Remove the timer if it doesn't repeat
	if (!timer.repeats) {
		[_timers removeObject:timer];
        [_runLoopTimersToTimers removeObjectForKey:runLoopTimer];
	}
}

- (CXTimer *)scheduleTimerWithInterval:(NSTimeInterval)interval repeats:(BOOL)repeats block:(void (^)(void))block {
	if (block == nil) {
		return nil;
	}
	
	CXTimer *timer = [CXTimer timerWithInterval:interval repeats:repeats block:block];
	[timer setDirector:self];
	[_timers addObject:timer];
	
	if (!_paused) {
		NSTimer *runLoopTimer = [NSTimer scheduledTimerWithTimeInterval:interval target:self selector:@selector(executeTimer:) userInfo:nil repeats:timer.repeats];
		timer.runLoopTimer = runLoopTimer;
		[_runLoopTimersToTimers setObject:timer forKey:runLoopTimer];
	}
	else {
		timer.nextInterval = interval;
	}
	
	return timer;
}

- (void)invalidateTimer:(CXTimer *)timer {
	if (timer == nil) {
		return;
	}
	
	[timer setDirector:nil];
	[_timers removeObject:timer];
	
	NSTimer *runLoopTimer = [timer runLoopTimer];
	if (runLoopTimer != nil) {
		[runLoopTimer invalidate];
		[_runLoopTimersToTimers removeObjectForKey:runLoopTimer];
	}
}

- (void)invalidateAllTimers {
	for (CXTimer *timer in _timers) {
		[self invalidateTimer:timer];
	}
}


#pragma mark -
#pragma mark Scenes

- (NSArray *)scenes {
	return [[_scenes array] copy];
}

- (CXScene *)currentScene {
	return [_scenes lastObject];
}

- (void)setCurrentScene:(CXScene *)currentScene {
	if ([_scenes containsObject:currentScene]) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene is already in the scene stack." userInfo:nil];
	}
	if (currentScene == nil) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene cannot be nil." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *previousScene = [_scenes lastObject];
	
    [previousScene sceneWillEnd];
	[currentScene sceneWillStart];
	
    if (previousScene != nil) {
        [_scenes removeObject:previousScene];
        previousScene.director = nil;
    }
    
	[_scenes addObject:currentScene];
    currentScene.director = self;
	
	CALayer *sceneLayer = [currentScene layer];
	sceneLayer.frame = self.baseLayer.bounds;
    if (previousScene != nil) {
        [self.baseLayer replaceSublayer:[previousScene layer] with:sceneLayer];
    }
    else {
        [self.baseLayer addSublayer:sceneLayer];
    }
	
    [previousScene sceneDidEnd];
	[currentScene sceneDidStart];
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
}

- (void)pushScene:(CXScene *)scene {
	if ([_scenes containsObject:scene]) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene is already in the scene stack." userInfo:nil];
	}
	if (scene == nil) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene cannot be nil." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *currentScene = [self currentScene];
	
    [currentScene sceneWillPause];
	[scene sceneWillStart];
	
	[_scenes addObject:scene];
    scene.director = self;
    
	CXLayer *sceneLayer = [scene layer];
	sceneLayer.frame = self.baseLayer.bounds;
    if (currentScene != nil) {
        [self.baseLayer replaceSublayer:[currentScene layer] with:sceneLayer];
    }
    else {
        [self.baseLayer addSublayer:sceneLayer];
    }
	
    [currentScene sceneDidPause];
	[scene sceneDidStart];
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
}

- (void)popScene {
	if ([_scenes count] == 0) {
		@throw [NSException exceptionWithName:NSRangeException reason:@"There are no scenes in the scene stack to pop." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeRemoval valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *currentScene = [self currentScene];
	[[currentScene layer] removeFromSuperlayer];
	
	CXScene *nextScene = [_scenes lastObject];
    
	[currentScene sceneWillEnd];
    [nextScene sceneWillResume];
    
    if (nextScene != nil) {
		CALayer *sceneLayer = [nextScene layer];
		sceneLayer.frame = self.baseLayer.bounds;
		[self.baseLayer replaceSublayer:[currentScene layer] with:sceneLayer];
	}
    else {
        [[currentScene layer] removeFromSuperlayer];
    }
    
	[_scenes removeObject:currentScene];
    currentScene.director = nil;
    
	[currentScene sceneDidEnd];
    [nextScene sceneDidResume];
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeRemoval valuesAtIndexes:indexes forKey:@"scenes"];
}

@end
