//
//  CXEvent.h
//  Chroniton
//
//  Created by Logan Collins on 4/16/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Chroniton/Chroniton.h>


/*!
 * @enum CXEventType
 * @abstract Event types
 * 
 * @constant CXEventTypeTouches		Touches on the screen
 * @constant CXEventTypeMouse		Movement and clicking of the mouse
 * @constant CXEventTypeKeyboard	Typing on the keyboard
 * @constant CXEventTypeMotion		Motion of the device
 */
enum {
	CXEventTypeTouches = 0,
	CXEventTypeMouse,
	CXEventTypeKeyboard,
	CXEventTypeMotion,
};
typedef NSUInteger CXEventType;


/*!
 * @enum CXEventSubtype
 * @abstract Event subtypes
 * 
 * @constant CXEventSubtypeLeftMouseDown		Left mouse button is pressed
 * @constant CXEventSubtypeLeftMouseDragged		Mouse is moved with the left button pressed
 * @constant CXEventSubtypeLeftMouseUp			Left mouse button is released
 * 
 * @constant CXEventSubtypeRightMouseDown		Right mouse button is pressed
 * @constant CXEventSubtypeRightMouseDragged	Mouse is moved with the right button pressed
 * @constant CXEventSubtypeRightMouseUp			Right mouse button is released
 * 
 * @constant CXEventSubtypeMouseMoved			Mouse is moved with no buttons pressed
 * 
 * @constant CXEventSubtypeKeyDown				Key is pressed on the keyboard
 * @constant CXEventSubtypeKeyUp				Key is released on the keyboard
 * 
 * @constant CXEventSubtypeMotionShake			Device is shaken
 */
enum {
	// Mouse
	CXEventSubtypeLeftMouseDown			= 1,
	CXEventSubtypeLeftMouseDragged		= 2,
	CXEventSubtypeLeftMouseUp			= 3,
	
	CXEventSubtypeRightMouseDown		= 4,
	CXEventSubtypeRightMouseDragged		= 5,
	CXEventSubtypeRightMouseUp			= 6,
	
	CXEventSubtypeMouseMoved			= 7,
	
	// Keyboard
	CXEventSubtypeKeyDown				= 100,
	CXEventSubtypeKeyUp					= 101,
	
	// Motion
	CXEventSubtypeMotionShake			= 200,
};
typedef NSUInteger CXEventSubtype;


/*!
 * @enum CXKey
 * @abstract Keys
 * 
 * @constant CXKeyReturn		Return
 * @constant CXKeyTab			Tab
 * @constant CXKeySpace			Space
 * @constant CXKeyDelete		Delete
 * @constant CXKeyEscape		Escape
 * @constant CXKeyCommand		Command
 * @constant CXKeyShift			Shift
 * @constant CXKeyCapsLock		CapsLock
 * @constant CXKeyOption		Option
 * @constant CXKeyControl		Control
 * @constant CXKeyFunction		Function
 * @constant CXKeyVolumeUp		Volume up
 * @constant CXKeyVolumeDown	Volume down
 * @constant CXKeyMute			Mute
 * @constant CXKeyHelp			Help
 * @constant CXKeyHome			Home
 * @constant CXKeyEnd			End
 * @constant CXKeyPageUp		Page up
 * @constant CXKeyPageDown		Page down
 * @constant CXKeyForwardDelete	Forward delete
 * @constant CXKeyLeftArrow		Left arrow
 * @constant CXKeyRightArrow	Right arrow
 * @constant CXKeyDownArrow		Down arrow
 * @constant CXKeyUpArrow		Up arrow
 */
enum {
    CXKeyReturn         = 0x24,
    CXKeyTab            = 0x30,
    CXKeySpace          = 0x31,
    CXKeyDelete         = 0x33,
    CXKeyEscape         = 0x35,
    CXKeyCommand        = 0x37,
    CXKeyShift          = 0x38,
    CXKeyCapsLock       = 0x39,
    CXKeyOption         = 0x3A,
    CXKeyControl        = 0x3B,
    CXKeyFunction       = 0x3F,
    CXKeyVolumeUp       = 0x48,
    CXKeyVolumeDown     = 0x49,
    CXKeyMute           = 0x4A,
    CXKeyHelp           = 0x72,
    CXKeyHome           = 0x73,
    CXKeyEnd            = 0x77,
    CXKeyPageUp         = 0x74,
    CXKeyPageDown       = 0x79,
    CXKeyForwardDelete  = 0x75,
    CXKeyLeftArrow      = 0x7B,
    CXKeyRightArrow     = 0x7C,
    CXKeyDownArrow      = 0x7D,
    CXKeyUpArrow        = 0x7E,
};
typedef NSUInteger CXKeyCode;


/*!
 * @class CXEvent
 * @abstract An event generated from user input
 */
@interface CXEvent : NSObject

/*!
 * @property director
 * @abstract The director that caught the event
 */
@property (strong, readonly) CXDirector *director;


/*!
 * @property timestamp
 * @abstract The time that the event occurred
 * 
 * @result An NSTimeInterval value
 */
@property (readonly) NSTimeInterval timestamp;

/*!
 * @property type
 * @abstract The type of the event
 * 
 * @result A CXEventType value
 */
@property (readonly) CXEventType type;

/*!
 * @property subtype
 * @abstract The subtype of the event
 * 
 * @result A CXEventSubtype value
 */
@property (readonly) CXEventType subtype;


/*!
 * @property touches
 * @abstract The touches for the event
 * 
 * @discussion
 * Only valid for touches-type events.
 * 
 * @result An NSSet of UITouch objects
 */
@property (copy, readonly) NSSet *touches;


/*!
 * @property location
 * @abstract The location the event was generated
 * 
 * @discussion
 * Only valid for mouse-type events.
 * 
 * @result A CGPoint value
 */
@property (readonly) CGPoint location;

/*!
 * @property clickCount
 * @abstract The number of mouse clicks
 * 
 * @discussion
 * Only valid for mouse-type events.
 * 
 * @result An NSUInteger value
 */
@property (readonly) NSUInteger clickCount;


/*!
 * @property keyCode
 * @abstract The key code of the key
 * 
 * @result An NSUInteger value
 */
@property (readonly) CXKeyCode keyCode;

/*!
 * @property repeat
 * @abstract The key is a repeat
 * 
 * @result A BOOL value
 */
@property (readonly, getter=isRepeat) NSUInteger repeat;

@end

