//
//  CXBezierPath.h
//  Chroniton
//
//  Created by Logan Collins on 5/2/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#endif


/*!
 * @class CXBezierPath
 * @abstract A high-level representation of a bezier path
 */
@interface CXBezierPath : NSObject <NSCopying>

/*!
 * @method bezierPath
 * @abstract Creates a new empty bezier path
 * 
 * @result A CXBezierPath object
 */
+ (CXBezierPath *)bezierPath;

/*!
 * @method bezierPathWithRect:
 * @abstract Creates a new bezier path with a rectangle
 * 
 * @param rect
 * The rect from which to create the path
 * 
 * @result A CXBezierPath object
 */
+ (CXBezierPath *)bezierPathWithRect:(CGRect)rect;

/*!
 * @method bezierPathWithRect:
 * @abstract Creates a new bezier path with an oval from a rectangle
 * 
 * @param rect
 * The rect from which to create an oval for the path
 * 
 * @result A CXBezierPath object
 */
+ (CXBezierPath *)bezierPathWithOvalInRect:(CGRect)rect;

/*!
 * @method bezierPathWithRoundedRect:cornerRadius:
 * @abstract Creates a new bezier path with a rounded rectangle
 * 
 * @param rect
 * The rect from which to create the path
 * 
 * @param cornerRadius
 * The radius of the rounded corners
 * 
 * @result A CXBezierPath object
 */
+ (CXBezierPath *)bezierPathWithRoundedRect:(CGRect)rect cornerRadius:(CGFloat)cornerRadius;

/*!
 * @method bezierPathWithCGPath:
 * @abstract Creates a new bezier path from a CGPath object
 * 
 * @param cgPath
 * The CGPath object from which to create the path
 * 
 * @result A CXBezierPath object
 */
+ (CXBezierPath *)bezierPathWithCGPath:(CGPathRef)cgPath;


/*!
 * @method moveToPath:
 * @abstract Moves the path's current point
 * 
 * @param point
 * The point to which to move
 */
- (void)moveToPath:(CGPoint)point;

/*!
 * @method addLineToPoint:
 * @abstract Appends a line segment to the path
 * 
 * @param point
 * The end point of the line
 */
- (void)addLineToPoint:(CGPoint)point;

/*!
 * @method addArcWithCenter:radius:startAngle:endAngle:clockwise:
 * @abstract Appends an arc to the path
 * 
 * @param center
 * The center point of the arc
 * 
 * @param radius
 * The radius of the arc
 * 
 * @param startAngle
 * The starting angle of the arc
 * 
 * @param endAngle
 * The ending angle of the arc
 * 
 * @param clockwise
 * Whether the angle is drawn clockwise
 */
- (void)addArcWithCenter:(CGPoint)center radius:(CGFloat)radius startAngle:(CGFloat)startAngle endAngle:(CGFloat)endAngle clockwise:(BOOL)clockwise;

/*!
 * @method addCurveToPoint:controlPoint1:controlPoint2:
 * @abstract Appends a bezier curve to the path
 * 
 * @param endPoint
 * The end point of the curve
 * 
 * @param controlPoint1
 * The first control point of the curve
 * 
 * @param controlPoint2
 * The second control point of the curve
 */
- (void)addCurveToPoint:(CGPoint)endPoint controlPoint1:(CGPoint)controlPoint1 controlPoint2:(CGPoint)controlPoint2;

/*!
 * @method addQuadCurveToPoint:controlPoint:
 * @abstract Appends a quadratic curve to the path
 * 
 * @param endPoint
 * The end point of the curve
 * 
 * @param controlPoint
 * The control point of the curve
 */
- (void)addQuadCurveToPoint:(CGPoint)endPoint controlPoint:(CGPoint)controlPoint;

/*!
 * @method closePath
 * @abstract Closes the most recent subpath of the path
 */
- (void)closePath;

/*!
 * @method removeAllPoints
 * @abstract Removes all points from the path, emptying it
 */
- (void)removeAllPoints;

/*!
 * @method appendPath:
 * @abstract Appends another path to the path
 * 
 * @param path
 * The path to append
 */
- (void)appendPath:(CXBezierPath *)path;

/*!
 * @method applyTransform:
 * @abstract Transforms all points in the path
 * 
 * @param transform
 * The transform to apply to the path
 */
- (void)applyTransform:(CGAffineTransform)transform;


/*!
 * @property CGPath
 * @abstract The CGPath representation of the path
 * 
 * @result A CGPathRef object
 */
@property (nonatomic) CGPathRef CGPath;

/*!
 * @property currentPoint
 * @abstract The current point of the path's most recent subpath
 * 
 * @result A CGPoint value
 */
@property (nonatomic, readonly) CGPoint currentPoint;


/*!
 * @property lineWidth
 * @abstract The line width
 * 
 * @discussion
 * The default line width is 1.0.
 * 
 * @result A CGFloat value
 */
@property (nonatomic) CGFloat lineWidth;

/*!
 * @property lineCapStyle
 * @abstract The line cap style
 * 
 * @discussion
 * The default line width is kCGLineCapButt.
 * 
 * @result A CGLineCap value
 */
@property (nonatomic) CGLineCap lineCapStyle;

/*!
 * @property lineJoinStyle
 * @abstract The line join style
 * 
 * @discussion
 * The default line width is kCGLineJoinMiter.
 * 
 * @result A CGLineJoin value
 */
@property (nonatomic) CGLineJoin lineJoinStyle;

/*!
 * @property miterLimit
 * @abstract The miter limit
 * 
 * @discussion
 * The default miter limit is 10.0.
 * 
 * @result A CGFloat value
 */
@property (nonatomic) CGFloat miterLimit;

/*!
 * @property flatness
 * @abstract The flatness
 * 
 * @discussion
 * The default line width is 0.6.
 * 
 * @result A CGFloat value
 */
@property (nonatomic) CGFloat flatness;

/*!
 * @property usesEvenOddFillRule
 * @abstract Whether the even-odd fill rule is used when drawing
 * 
 * @discussion
 * The default is NO.
 * 
 * @result A BOOL value
 */
@property (nonatomic) BOOL usesEvenOddFillRule;

/*!
 * @method getLineDash:count:phase:
 * @abstract Gets the stroke line dash style
 * 
 * @param pattern
 * On input, a pointer to an array of CGFloat values large enough to hold 'count' values.
 * On output, the values for the line dash pattern.
 * 
 * @param count
 * On input, a pointer to an NSInteger value. On output, the number of values in 'pattern'.
 * 
 * @param phase
 * On input, a pointer to a CGFloat value. On output, the value of the line dash phase.
 * 
 * @discussion
 * If the number of pattern components is not known, pass NULL for pattern. The value of
 * 'count' can then be used to call this method again.
 */
- (void)getLineDash:(CGFloat *)pattern count:(NSInteger *)count phase:(CGFloat *)phase;

/*!
 * @method setLineDash:count:phase:
 * @abstract Sets the stroke line dash style
 * 
 * @param pattern
 * An array of CGFloat values
 * 
 * @param
 * The number of values in 'pattern'
 * 
 * @param phase
 * The phase of the line dash
 */
- (void)setLineDash:(const CGFloat *)pattern count:(NSInteger)count phase:(CGFloat)phase;


/*!
 * @property empty
 * @abstract Whether the path contains any points
 * 
 * @discussion
 * A path is not considered empty if moveToPoint: has been called on it.
 * 
 * @result A BOOL value
 */
@property (nonatomic, readonly, getter=isEmpty) BOOL empty;

/*!
 * @property bounds
 * @abstract The bounding rectangle of the path
 * 
 * @discussion
 * This is the smallest rect such that all points of the path are contained
 * 
 * @result A CGRect value
 */
@property (nonatomic, readonly) CGRect bounds;

/*!
 * @method containsPoint:
 * @abstract Test to determine if a point is contained in the path
 * 
 * @param point
 * The point to test
 * 
 * @result A BOOL value
 */
- (BOOL)containsPoint:(CGPoint)point;


/*!
 * @method fill
 * @abstract Fills the path using the current graphics fill color
 * 
 * @discussion
 * This is equivalent to calling fillWithBlendMode:alpha: with a blend mode value
 * of kCGBlendModeNormal and an alpha value of 1.0.
 */
- (void)fill;

/*!
 * @method fillWithBlendMode:alpha:
 * @abstract Fills the path using the current graphics fill color
 * 
 * @param blendMode
 * The blend mode to use when drawing
 * 
 * @param alpha
 * The alpha value to use when drawing
 */
- (void)fillWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha;

/*!
 * @method stroke
 * @abstract Strokes the path using the current graphics stroke color
 * 
 * @discussion
 * This is equivalent to calling strokeWithBlendMode:alpha: with a blend mode value
 * of kCGBlendModeNormal and an alpha value of 1.0.
 */
- (void)stroke;

/*!
 * @method strokeWithBlendMode:alpha:
 * @abstract Strokes the path using the current graphics stroke color
 * 
 * @param blendMode
 * The blend mode to use when drawing
 * 
 * @param alpha
 * The alpha value to use when drawing
 */
- (void)strokeWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha;


/*!
 * @method addClip
 * @abstract Adds the path to the current graphics clipping mask
 */
- (void)addClip;

@end
