//
//  CXBezierPath.m
//  Chroniton
//
//  Created by Logan Collins on 5/2/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXBezierPath.h"
#import "CXGraphics.h"


@interface CXBezierPath ()

- (id)initWithCGPath:(CGPathRef)cgPath;

@end


@implementation CXBezierPath {
    CGMutablePathRef _path;
    CGPathRef _immutablePath;
    BOOL _immutablePathIsValid;
    CGFloat *_lineDashPattern;
    CGFloat _lineDashPatternCount;
    CGFloat _lineDashPhase;
}

@synthesize lineWidth=_lineWidth;
@synthesize lineCapStyle=_lineCapStyle;
@synthesize lineJoinStyle=_lineJoinStyle;
@synthesize miterLimit=_miterLimit;
@synthesize flatness=_flatness;
@synthesize usesEvenOddFillRule=_usesEvenOddFillRule;

+ (CXBezierPath *)bezierPath {
    return [self bezierPathWithCGPath:NULL];
}

+ (CXBezierPath *)bezierPathWithRect:(CGRect)rect {
    CGPathRef path = CGPathCreateWithRect(rect, NULL);
    CXBezierPath *bezierPath = [self bezierPathWithCGPath:path];
    CGPathRelease(path);
    return bezierPath;
}

+ (CXBezierPath *)bezierPathWithOvalInRect:(CGRect)rect {
    CGPathRef path = CGPathCreateWithEllipseInRect(rect, NULL);
    CXBezierPath *bezierPath = [self bezierPathWithCGPath:path];
    CGPathRelease(path);
    return bezierPath;
}

+ (CXBezierPath *)bezierPathWithRoundedRect:(CGRect)rect cornerRadius:(CGFloat)cornerRadius {
    CGMutablePathRef path = CGPathCreateMutable();
    
	CGRect innerRect = CGRectInset(rect, cornerRadius, cornerRadius);
    
	CGFloat insideRight = innerRect.origin.x + innerRect.size.width;
	CGFloat outsideRight = rect.origin.x + rect.size.width;
	CGFloat insideBottom = innerRect.origin.y + innerRect.size.height;
	CGFloat outsideBottom = rect.origin.y + rect.size.height;
    
	CGFloat insideTop = innerRect.origin.y;
	CGFloat outsideTop = rect.origin.y;
	CGFloat outsideLeft = rect.origin.x;
    
	CGPathMoveToPoint(path, NULL, innerRect.origin.x, outsideTop);
    
	CGPathAddLineToPoint(path, NULL, insideRight, outsideTop);
	CGPathAddArcToPoint(path, NULL, outsideRight, outsideTop, outsideRight, insideTop, cornerRadius);
	CGPathAddLineToPoint(path, NULL, outsideRight, insideBottom);
	CGPathAddArcToPoint(path, NULL,  outsideRight, outsideBottom, insideRight, outsideBottom, cornerRadius);
    
	CGPathAddLineToPoint(path, NULL, innerRect.origin.x, outsideBottom);
	CGPathAddArcToPoint(path, NULL,  outsideLeft, outsideBottom, outsideLeft, insideBottom, cornerRadius);
	CGPathAddLineToPoint(path, NULL, outsideLeft, insideTop);
	CGPathAddArcToPoint(path, NULL,  outsideLeft, outsideTop, innerRect.origin.x, outsideTop, cornerRadius);
    
	CGPathCloseSubpath(path);
    
    CXBezierPath *bezierPath = [self bezierPathWithCGPath:path];
    
    CGPathRelease(path);
    
    return bezierPath;
}

+ (CXBezierPath *)bezierPathWithCGPath:(CGPathRef)cgPath {
    return [[self alloc] initWithCGPath:cgPath];
}

- (id)initWithCGPath:(CGPathRef)cgPath {
    self = [super init];
    if (self) {
        if (cgPath != NULL) {
            _path = CGPathCreateMutableCopy(cgPath);
        }
        else {
            _path = CGPathCreateMutable();
        }
        
        _lineWidth = 1.0;
        _lineCapStyle = kCGLineCapButt;
        _lineJoinStyle = kCGLineJoinMiter;
        _miterLimit = 10.0;
        _flatness = 0.6;
        _usesEvenOddFillRule = NO;
    }
    return self;
}

- (id)init {
    return [self initWithCGPath:NULL];
}

- (id)copyWithZone:(NSZone *)zone {
    CXBezierPath *bezierPath = [[[self class] alloc] initWithCGPath:_path];
    
    bezierPath.lineWidth = self.lineWidth;
    bezierPath.lineCapStyle = self.lineCapStyle;
    bezierPath.lineJoinStyle = self.lineJoinStyle;
    bezierPath.miterLimit = self.miterLimit;
    bezierPath.flatness = self.flatness;
    bezierPath.usesEvenOddFillRule = self.usesEvenOddFillRule;
    
    if (_lineDashPattern != NULL) {
        CGFloat * lineDashPattern = (CGFloat *)malloc(sizeof(CGFloat) * _lineDashPatternCount);
        for (NSUInteger i=0; i<_lineDashPatternCount; i++) {
            lineDashPattern[i] = _lineDashPattern[i];
        }
        bezierPath->_lineDashPattern = lineDashPattern;
        bezierPath->_lineDashPatternCount = _lineDashPatternCount;
        bezierPath->_lineDashPhase = _lineDashPhase;
    }
    
    return bezierPath;
}

- (void)dealloc {
    CGPathRelease(_path);
    if (_lineDashPattern != NULL) {
        free(_lineDashPattern);
    }
    CGPathRelease(_immutablePath);
}

- (NSUInteger)hash {
    return CFHash(_path);
}

- (BOOL)isEqual:(id)object {
    if ([object isKindOfClass:[CXBezierPath class]]) {
        return CFEqual(_path, ((CXBezierPath *)object)->_path);
    }
    return NO;
}


#pragma mark -
#pragma mark Accessors

- (CGPathRef)CGPath {
    if (_immutablePathIsValid == NO || _immutablePath == NULL) {
        CGPathRelease(_immutablePath);
        _immutablePath = CGPathCreateCopy(_path);
        _immutablePathIsValid = YES;
    }
    return _immutablePath;
}

- (void)setCGPath:(CGPathRef)cgPath {
    _immutablePathIsValid = NO;
    
    CGPathRelease(_path);
    if (cgPath != NULL) {
        _path = CGPathCreateMutableCopy(cgPath);
    }
    else {
        _path = CGPathCreateMutable();
    }
}

- (CGPoint)currentPoint {
    return CGPathGetCurrentPoint(_path);
}

- (void)getLineDash:(CGFloat *)pattern count:(NSInteger *)count phase:(CGFloat *)phase {
    if (pattern != NULL) {
        for (NSUInteger i=0; i<_lineDashPatternCount; i++) {
            pattern[i] = _lineDashPattern[i];
        }
    }
    if (count != NULL) {
        *count = _lineDashPatternCount;
    }
    if (phase != NULL) {
        *phase = _lineDashPhase;
    }
}

- (void)setLineDash:(const CGFloat *)pattern count:(NSInteger)count phase:(CGFloat)phase {
    if (_lineDashPattern != NULL) {
        free(_lineDashPattern);
    }
    
    if (count > 0) {
        _lineDashPattern = (CGFloat *)malloc(sizeof(CGFloat) * count);
        
        for (NSUInteger i=0; i<count; i++) {
            _lineDashPattern[i] = pattern[i];
        }
        
        _lineDashPatternCount = count;
        _lineDashPhase = phase;
    }
}


#pragma mark -
#pragma mark Construction

- (void)moveToPath:(CGPoint)point {
    _immutablePathIsValid = NO;
    
    CGPathMoveToPoint(_path, NULL, point.x, point.y);
}

- (void)addLineToPoint:(CGPoint)point {
    _immutablePathIsValid = NO;
    
    CGPathAddLineToPoint(_path, NULL, point.x, point.y);
}

- (void)addArcWithCenter:(CGPoint)center radius:(CGFloat)radius startAngle:(CGFloat)startAngle endAngle:(CGFloat)endAngle clockwise:(BOOL)clockwise {
    _immutablePathIsValid = NO;
    
    CGPathAddArc(_path, NULL, center.x, center.y, radius, startAngle, endAngle, clockwise);
}

- (void)addCurveToPoint:(CGPoint)endPoint controlPoint1:(CGPoint)controlPoint1 controlPoint2:(CGPoint)controlPoint2 {
    _immutablePathIsValid = NO;
    
    CGPathAddCurveToPoint(_path, NULL, controlPoint1.x, controlPoint1.y, controlPoint2.x, controlPoint2.y, endPoint.x, endPoint.y);
}

- (void)addQuadCurveToPoint:(CGPoint)endPoint controlPoint:(CGPoint)controlPoint {
    _immutablePathIsValid = NO;
    
    CGPathAddQuadCurveToPoint(_path, NULL, controlPoint.x, controlPoint.y, endPoint.x, endPoint.y);
}

- (void)closePath {
    _immutablePathIsValid = NO;
    
    CGPathCloseSubpath(_path);
}

- (void)removeAllPoints {
    _immutablePathIsValid = NO;
    
    CGPathRelease(_path);
    _path = CGPathCreateMutable();
}

- (void)appendPath:(CXBezierPath *)path {
    _immutablePathIsValid = NO;
    
    CGPathAddPath(_path, NULL, [path CGPath]);
}

- (void)applyTransform:(CGAffineTransform)transform {
    _immutablePathIsValid = NO;
    
    CGMutablePathRef newPath = CGPathCreateMutableCopyByTransformingPath(_path, (const CGAffineTransform *)&transform);
    CGPathRelease(_path);
    _path = newPath;
}


#pragma mark -
#pragma mark Hit Testing

- (BOOL)isEmpty {
    return CGPathIsEmpty(_path);
}

- (CGRect)bounds {
    return CGPathGetBoundingBox(_path);
}

- (BOOL)containsPoint:(CGPoint)point {
    return CGPathContainsPoint(_path, NULL, point, self.usesEvenOddFillRule);
}


#pragma mark -
#pragma mark Drawing

- (void)fill {
    [self fillWithBlendMode:kCGBlendModeNormal alpha:1.0];
}

- (void)fillWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha {
    CGContextRef c = CXGraphicsGetCurrentContext();
    CGContextSaveGState(c);
    
    CGContextAddPath(c, _path);
    
    CGContextSetLineWidth(c, self.lineWidth);
    CGContextSetLineCap(c, self.lineCapStyle);
    CGContextSetLineJoin(c, self.lineJoinStyle);
    CGContextSetMiterLimit(c, self.miterLimit);
    CGContextSetFlatness(c, self.flatness);
    if (_lineDashPattern != NULL) {
        CGContextSetLineDash(c, _lineDashPhase, _lineDashPattern, _lineDashPatternCount);
    }
    
    CGContextSetBlendMode(c, blendMode);
    CGContextSetAlpha(c, alpha);
    
    if (self.usesEvenOddFillRule) {
        CGContextEOFillPath(c);
    }
    else {
        CGContextFillPath(c);
    }
    
    CGContextRestoreGState(c);
}

- (void)stroke {
    [self strokeWithBlendMode:kCGBlendModeNormal alpha:1.0];
}

- (void)strokeWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha {
    CGContextRef c = CXGraphicsGetCurrentContext();
    CGContextSaveGState(c);
    
    CGContextAddPath(c, _path);
    
    CGContextSetLineWidth(c, self.lineWidth);
    CGContextSetLineCap(c, self.lineCapStyle);
    CGContextSetLineJoin(c, self.lineJoinStyle);
    CGContextSetMiterLimit(c, self.miterLimit);
    CGContextSetFlatness(c, self.flatness);
    if (_lineDashPattern != NULL) {
        CGContextSetLineDash(c, _lineDashPhase, _lineDashPattern, _lineDashPatternCount);
    }
    
    CGContextSetBlendMode(c, blendMode);
    CGContextSetAlpha(c, alpha);
    
    CGContextStrokePath(c);
    
    CGContextRestoreGState(c);
}

- (void)addClip {
    CGContextRef c = CXGraphicsGetCurrentContext();
    
    CGContextAddPath(c, _path);
    
    CGContextSetLineWidth(c, self.lineWidth);
    CGContextSetLineCap(c, self.lineCapStyle);
    CGContextSetLineJoin(c, self.lineJoinStyle);
    CGContextSetMiterLimit(c, self.miterLimit);
    CGContextSetFlatness(c, self.flatness);
    if (_lineDashPattern != NULL) {
        CGContextSetLineDash(c, _lineDashPhase, _lineDashPattern, _lineDashPatternCount);
    }
    
    if (self.usesEvenOddFillRule) {
        CGContextEOClip(c);
    }
    else {
        CGContextClip(c);
    }
}

@end
