//
//  CXColor.h
//  Chroniton
//
//  Created by Logan Collins on 4/6/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#import <CoreImage/CoreImage.h>
#elif TARGET_OS_MAC
#import <QuartzCore/QuartzCore.h>
#endif


/*!
 * @class CXColor
 * @abstract A high-level representation of a color
 */
@interface CXColor : NSObject <NSCoding>

/*!
 * @method colorWithHue:saturation:brightness:alpha:
 * @abstract Creates a new color using HSB values
 * 
 * @param hue
 * The hue value
 * 
 * @param saturation
 * The saturation value
 * 
 * @param brightness
 * The brightness value
 * 
 * @param alpha
 * The alpha value
 * 
 * @result A CXColor object
 */
+ (CXColor *)colorWithHue:(CGFloat)hue saturation:(CGFloat)saturation brightness:(CGFloat)brightness alpha:(CGFloat)alpha;

/*!
 * @method colorWithRed:green:blue:alpha:
 * @abstract Creates a new color using RGB values
 * 
 * @param red
 * The red value
 * 
 * @param green
 * The green value
 * 
 * @param blue
 * The blue value
 * 
 * @param alpha
 * The alpha value
 * 
 * @result A CXColor object
 */
+ (CXColor *)colorWithRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue alpha:(CGFloat)alpha;

/*!
 * @method colorWithWhite:alpha:
 * @abstract Creates a new color using White values
 * 
 * @param white
 * The white value
 * 
 * @param alpha
 * The alpha value
 * 
 * @result A CXColor object
 */
+ (CXColor *)colorWithWhite:(CGFloat)white alpha:(CGFloat)alpha;

/*!
 * @method colorWithCGColor:
 * @abstract Creates a new color from a CGColor object
 * 
 * @param cgColor
 * The CGColor object
 * 
 * @result A CXColor object
 */
+ (CXColor *)colorWithCGColor:(CGColorRef)cgColor;

/*!
 * @method colorWithCIColor:
 * @abstract Creates a new color from a CIColor object
 * 
 * @param ciColor
 * The CIColor object
 * 
 * @result A CXColor object
 */
+ (CXColor *)colorWithCIColor:(CIColor *)ciColor;


/*!
 * @method whiteColor
 * @abstract Creates a new color with a white value of 1.0 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)whiteColor;

/*!
 * @method blackColor
 * @abstract Creates a new color with a white value of 0.0 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)blackColor;

/*!
 * @method grayColor
 * @abstract Creates a new color with a white value of 0.5 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)grayColor;

/*!
 * @method lightGrayColor
 * @abstract Creates a new color with a white value of 2/3 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)lightGrayColor;

/*!
 * @method darkGrayColor
 * @abstract Creates a new color with a white value of 1/3 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)darkGrayColor;

/*!
 * @method grayColor
 * @abstract Creates a new color with a white value of 0.5 and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)grayColor;

/*!
 * @method redColor
 * @abstract Creates a new color with an RGB value of (1.0, 0.0, 0.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)redColor;

/*!
 * @method greenColor
 * @abstract Creates a new color with an RGB value of (0.0, 1.0, 0.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)greenColor;

/*!
 * @method blueColor
 * @abstract Creates a new color with an RGB value of (0.0, 0.0, 1.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)blueColor;

/*!
 * @method cyanColor
 * @abstract Creates a new color with an RGB value of (0.0, 1.0, 1.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)cyanColor;

/*!
 * @method magentaColor
 * @abstract Creates a new color with an RGB value of (1.0, 0.0, 1.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)magentaColor;

/*!
 * @method yellowColor
 * @abstract Creates a new color with an RGB value of (1.0, 1.0, 0.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)yellowColor;

/*!
 * @method orangeColor
 * @abstract Creates a new color with an RGB value of (1.0, 0.5, 0.0) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)orangeColor;

/*!
 * @method brownColor
 * @abstract Creates a new color with an RGB value of (0.6, 0.4, 0.2) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)brownColor;

/*!
 * @method purpleColor
 * @abstract Creates a new color with an RGB value of (0.5, 0.0, 0.5) and an alpha value of 1.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)purpleColor;

/*!
 * @method clearColor
 * @abstract Creates a new color with a white value value of 0.0 and an alpha value of 0.0
 * 
 * @result A CXColor object
 */
+ (CXColor *)clearColor;


/*!
 * @property CGColor
 * @abstract Gets a CGColor representation
 * 
 * @result A CGColor object
 */
@property (readonly) CGColorRef CGColor;

/*!
 * @property CIColor
 * @abstract Gets a CIColor representation
 * 
 * @result A CIColor object
 */
@property (readonly) CIColor *CIColor;


/*!
 * @method colorWithAlphaComponent:
 * @abstract Creates a copy of the color with a new alpha value
 * 
 * @param alpha
 * The alpha value
 * 
 * @result A CXColor object
 */
- (CXColor *)colorWithAlphaComponent:(CGFloat)alpha;


/*!
 * @method getHue:saturation:brightness:alpha:
 * @abstract Returns HSB values by reference
 * 
 * @param hue
 * Pointer to a CGFloat value to contain the hue value
 * 
 * @param saturation
 * Pointer to a CGFloat value to contain the saturation value
 * 
 * @param brightness
 * Pointer to a CGFloat value to contain the brightness value
 * 
 * @param alpha
 * Pointer to a CGFloat value to contain the alpha value
 * 
 * @result A BOOL value; YES if HSB values could be extracted, NO otherwise.
 */
- (BOOL)getHue:(CGFloat *)hue saturation:(CGFloat *)saturation brightness:(CGFloat *)brightness alpha:(CGFloat *)alpha;

/*!
 * @method getRed:green:blue:alpha:
 * @abstract Returns RGB values by reference
 * 
 * @param red
 * Pointer to a CGFloat value to contain the red value
 * 
 * @param green
 * Pointer to a CGFloat value to contain the green value
 * 
 * @param blue
 * Pointer to a CGFloat value to contain the blue value
 * 
 * @param alpha
 * Pointer to a CGFloat value to contain the alpha value
 * 
 * @result A BOOL value; YES if RGB values could be extracted, NO otherwise.
 */
- (BOOL)getRed:(CGFloat *)red green:(CGFloat *)green blue:(CGFloat *)blue alpha:(CGFloat *)alpha;

/*!
 * @method getWhite:alpha:
 * @abstract Returns White values by reference
 * 
 * @param white
 * Pointer to a CGFloat value to contain the white value
 * 
 * @param alpha
 * Pointer to a CGFloat value to contain the alpha value
 * 
 * @result A BOOL value; YES if RGB values could be extracted, NO otherwise.
 */
- (BOOL)getWhite:(CGFloat *)white alpha:(CGFloat *)alpha;


/*!
 * @method set
 * @abstract Sets the color as both the fill and stroke color in the current graphics context
 */
- (void)set;

/*!
 * @method setFill
 * @abstract Sets the color as the fill color in the current graphics context
 */
- (void)setFill;

/*!
 * @method setFill
 * @abstract Sets the color as the stroke color in the current graphics context
 */
- (void)setStroke;

@end
