//
//  CXColor.m
//  Chroniton
//
//  Created by Logan Collins on 4/6/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXColor.h"
#import "CXGraphics.h"


static void CXColorConvertRGBToHSB(CGFloat red, CGFloat green, CGFloat blue, CGFloat *hue, CGFloat *saturation, CGFloat *brightness);
static void CXColorConvertHSBToRGB(CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat *red, CGFloat *green, CGFloat *blue);


@implementation CXColor {
	CGColorRef _cgColor;
}

+ (CXColor *)colorWithWhite:(CGFloat)white alpha:(CGFloat)alpha {
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceGray();
    CGFloat components[2];
    components[0] = white;
    components[1] = alpha;
	CGColorRef cgColor = CGColorCreate(colorSpace, (const CGFloat *)&components);
	CXColor *color = [self colorWithCGColor:cgColor];
	CGColorRelease(cgColor);
    CGColorSpaceRelease(colorSpace);
	return color;
}

+ (CXColor *)colorWithHue:(CGFloat)hue saturation:(CGFloat)saturation brightness:(CGFloat)brightness alpha:(CGFloat)alpha {
	CGFloat red, green, blue;
	CXColorConvertHSBToRGB(hue, saturation, brightness, &red, &green, &blue);
	return [self colorWithRed:red green:green blue:blue alpha:alpha];
}

+ (CXColor *)colorWithRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue alpha:(CGFloat)alpha {
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGFloat components[4];
    components[0] = red;
    components[1] = green;
    components[2] = blue;
    components[3] = alpha;
	CGColorRef cgColor = CGColorCreate(colorSpace, (const CGFloat *)&components);
	CXColor *color = [self colorWithCGColor:cgColor];
	CGColorRelease(cgColor);
    CGColorSpaceRelease(colorSpace);
	return color;
}

+ (CXColor *)colorWithCGColor:(CGColorRef)cgColor {
	return [[self alloc] initWithCGColor:cgColor];
}

+ (CXColor *)colorWithCIColor:(CIColor *)ciColor {
	return [self colorWithRed:[ciColor red] green:[ciColor green] blue:[ciColor blue] alpha:[ciColor alpha]];
}

- (id)initWithCGColor:(CGColorRef)cgColor {
	if (cgColor == NULL) {
		return nil;
	}
	
	self = [super init];
	if (self) {
		_cgColor = CGColorCreateCopy(cgColor);
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	self = [super init];
	if (self) {
		CGColorSpaceModel colorSpaceModel = [aDecoder decodeIntegerForKey:@"CXColorSpaceModel"];
        CGColorSpaceRef colorSpace = NULL;
        if (colorSpaceModel == kCGColorSpaceModelMonochrome) {
            colorSpace = CGColorSpaceCreateDeviceGray();
        }
        else if (colorSpaceModel == kCGColorSpaceModelRGB) {
            colorSpace = CGColorSpaceCreateDeviceRGB();
        }
        
		if (colorSpace == NULL) {
			return nil;
		}
		
		size_t count = CGColorSpaceGetNumberOfComponents(colorSpace);
		CGFloat *components = (CGFloat *)malloc(sizeof(CGFloat) * count);
		for (size_t i=0; i<count; i++) {
			CGFloat value = [aDecoder decodeDoubleForKey:[NSString stringWithFormat:@"CXComponent%d", i]];
			components[i] = value;
		}
		
		CGColorRef color = CGColorCreate(colorSpace, (const CGFloat *)components);
		free(components);
		
		if (color == NULL) {
			return nil;
		}
		_cgColor = color;
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder {
	CGColorSpaceRef colorSpace = CGColorGetColorSpace(_cgColor);
	CGColorSpaceModel colorSpaceModel = CGColorSpaceGetModel(colorSpace);
    [aCoder encodeInteger:colorSpaceModel forKey:@"CXColorSpaceModel"];
	
	const CGFloat * components = CGColorGetComponents(_cgColor);
	size_t count = CGColorGetNumberOfComponents(_cgColor);
	for (size_t i=0; i<count; i++) {
		[aCoder encodeDouble:components[i] forKey:[NSString stringWithFormat:@"CXComponent%d", i]];
	}
}

- (void)dealloc {
	CFRelease(_cgColor);
}

- (NSUInteger)hash {
	return CFHash(_cgColor);
}

- (BOOL)isEqual:(id)object {
	if ([object isKindOfClass:[CXColor class]]) {
		return CFEqual(_cgColor, ((CXColor *)object)->_cgColor);
	}
	return NO;
}

- (CGColorRef)CGColor {
	return _cgColor;
}

- (CIColor *)CIColor {
	return [CIColor colorWithCGColor:_cgColor];
}

- (CXColor *)colorWithAlphaComponent:(CGFloat)alpha {
	CGColorRef cgColor = CGColorCreateCopyWithAlpha(_cgColor, alpha);
	CXColor *color = [CXColor colorWithCGColor:cgColor];
	CGColorRelease(cgColor);
	return color;
}

- (BOOL)getHue:(CGFloat *)hue saturation:(CGFloat *)saturation brightness:(CGFloat *)brightness alpha:(CGFloat *)alpha {
	CGFloat tempHue, tempSaturation, tempBrightness, tempAlpha;
	CGFloat red, green, blue;
	if (![self getRed:&red green:&green blue:&blue alpha:&tempAlpha]) {
		return NO;
	}
	CXColorConvertRGBToHSB(red, green, blue, &tempHue, &tempSaturation, &tempBrightness);
	if (hue != NULL) {
		*hue = tempHue;
	}
	if (saturation != NULL) {
		*saturation = tempSaturation;
	}
	if (brightness != NULL) {
		*brightness = tempBrightness;
	}
	return YES;
}

- (BOOL)getRed:(CGFloat *)red green:(CGFloat *)green blue:(CGFloat *)blue alpha:(CGFloat *)alpha {
	CGColorRef destColor = _cgColor;
	CGColorSpaceRef destColorSpace = CGColorSpaceCreateDeviceRGB();
	CGColorSpaceRef currentColorSpace = CGColorGetColorSpace(destColor);
	if (!CFEqual(destColorSpace, currentColorSpace)) {
		return NO;
	}
	CGColorSpaceRelease(destColorSpace);
	
	const CGFloat * values = CGColorGetComponents(destColor);
	if (red != NULL) {
		*red = values[0];
	}
	if (green != NULL) {
		*green = values[1];
	}
	if (blue != NULL) {
		*blue = values[2];
	}
	if (alpha != NULL) {
		*alpha = values[3];
	}
	
	return YES;
}

- (BOOL)getWhite:(CGFloat *)white alpha:(CGFloat *)alpha {
	CGColorRef destColor = _cgColor;
	CGColorSpaceRef destColorSpace = CGColorSpaceCreateDeviceGray();
	CGColorSpaceRef currentColorSpace = CGColorGetColorSpace(destColor);
	if (!CFEqual(destColorSpace, currentColorSpace)) {
		return NO;
	}
	CGColorSpaceRelease(destColorSpace);
	
	const CGFloat * values = CGColorGetComponents(destColor);
	if (white != NULL) {
		*white = values[0];
	}
	if (alpha != NULL) {
		*alpha = values[1];
	}
	
	return YES;
}

- (void)set {
	CGContextRef c = CXGraphicsGetCurrentContext();
	CGContextSetFillColorWithColor(c, _cgColor);
	CGContextSetStrokeColorWithColor(c, _cgColor);
}

- (void)setFill {
	CGContextRef c = CXGraphicsGetCurrentContext();
	CGContextSetFillColorWithColor(c, _cgColor);
}

- (void)setStroke {
	CGContextRef c = CXGraphicsGetCurrentContext();
	CGContextSetStrokeColorWithColor(c, _cgColor);
}


#pragma mark -
#pragma mark Built-in colors

+ (CXColor *)whiteColor {
    return [self colorWithWhite:1.0 alpha:1.0];
}

+ (CXColor *)blackColor {
    return [self colorWithWhite:0.0 alpha:1.0];
}

+ (CXColor *)grayColor {
    return [self colorWithWhite:0.5 alpha:1.0];
}

+ (CXColor *)lightGrayColor {
    return [self colorWithWhite:(2.0/3.0) alpha:1.0];
}

+ (CXColor *)darkGrayColor {
    return [self colorWithWhite:(1.0/3.0) alpha:1.0];
}

+ (CXColor *)redColor {
    return [self colorWithRed:1.0 green:0.0 blue:0.0 alpha:1.0];
}

+ (CXColor *)greenColor {
    return [self colorWithRed:0.0 green:1.0 blue:0.0 alpha:1.0];
}

+ (CXColor *)blueColor {
    return [self colorWithRed:0.0 green:0.0 blue:1.0 alpha:1.0];
}

+ (CXColor *)cyanColor {
    return [self colorWithRed:0.0 green:1.0 blue:1.0 alpha:1.0];
}

+ (CXColor *)magentaColor {
    return [self colorWithRed:1.0 green:0.0 blue:1.0 alpha:1.0];
}

+ (CXColor *)yellowColor {
    return [self colorWithRed:1.0 green:1.0 blue:0.0 alpha:1.0];
}

+ (CXColor *)orangeColor {
    return [self colorWithRed:1.0 green:0.5 blue:0.0 alpha:1.0];
}

+ (CXColor *)brownColor {
    return [self colorWithRed:0.6 green:0.4 blue:0.2 alpha:1.0];
}

+ (CXColor *)purpleColor {
    return [self colorWithRed:0.5 green:0.0 blue:0.5 alpha:1.0];
}

+ (CXColor *)clearColor {
    return [self colorWithWhite:0.0 alpha:0.0];
}

@end


static void CXColorConvertRGBToHSB(CGFloat red, CGFloat green, CGFloat blue, CGFloat *hue, CGFloat *saturation, CGFloat *brightness) {
	CGFloat min, max, delta;
	min = MIN(red, MIN(green, blue));
	max = MAX(red, MAX(green, blue));
	*brightness = max;
	delta = max - min;
	if (max != 0) {
		*saturation = delta / max;
	}
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*saturation = 0;
		*hue = -1.0;
		return;
	}
	if (red == max)
		*hue = (green - blue) / delta;		// between yellow & magenta
	else if (green == max)
		*hue = 2 + (blue - red) / delta;	// between cyan & yellow
	else
		*hue = 4 + (red - green) / delta;	// between magenta & cyan
	*hue *= 60.0;
	if (*hue < 0.0)
		*hue += 360.0;
	*hue /= 360.0;
}

static void CXColorConvertHSBToRGB(CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat *red, CGFloat *green, CGFloat *blue) {
	hue *= 360.0;
	
	NSInteger i;
	CGFloat f, p, q, t;
	
	if (saturation == 0) {
		// Achromatic (grey)
		*red = *green = *blue = brightness;
		return;
	}
	
	hue /= 60;			// sector 0 to 5
	i = floor(hue);
	f = hue - i;			// factorial part of h
	p = brightness * (1 - saturation);
	q = brightness * (1 - saturation * f);
	t = brightness * (1 - saturation * (1 - f));
	
	switch(i) {
		case 0:
			*red = brightness;
			*green = t;
			*blue = p;
			break;
		case 1:
			*red = q;
			*green = brightness;
			*blue = p;
			break;
		case 2:
			*red = p;
			*green = brightness;
			*blue = t;
			break;
		case 3:
			*red = p;
			*green = q;
			*blue = brightness;
			break;
		case 4:
			*red = t;
			*green = p;
			*blue = brightness;
			break;
		default:		// case 5:
			*red = brightness;
			*green = p;
			*blue = q;
			break;
	}
}
