//
//  CXGradient.h
//  Chroniton
//
//  Created by Logan Collins on 5/3/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>

#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#endif


/*!
 * @enum CXGradientDrawingOptions
 * @abstract Gradient drawing options
 * 
 * @constant CXGradientDrawsBeforeStartingLocation      The gradient draws before the starting location
 * @constant CXGradientDrawsAfterEndingLocation         The gradient draws after the ending location
 */
enum {
    CXGradientDrawsBeforeStartingLocation = (1 << 0),
    CXGradientDrawsAfterEndingLocation = (1 << 1),
};
typedef NSUInteger CXGradientDrawingOptions;


@class CXColor, CXBezierPath;


/*!
 * @class CXGradient
 * @abstract A high-level representation of a color gradient
 */
@interface CXGradient : NSObject <NSCopying>

/*!
 * @method gradientWithStartingColor:endingColor:
 * @abstract Creates a new gradient from two colors
 * 
 * @param startingColor
 * The starting color (location 0.0)
 * 
 * @param endingColor
 * The ending color (location 1.0)
 * 
 * @result A CXGradient object
 */
+ (CXGradient *)gradientWithStartingColor:(CXColor *)startingColor endingColor:(CXColor *)endingColor;

/*!
 * @method gradientWithColors:
 * @abstract Creates a new gradient from an array of colors
 * 
 * @param colors
 * The colors
 * 
 * @discussion
 * The colors are evenly spaced over the gradient.
 * 
 * @result A CXGradient object
 */
+ (CXGradient *)gradientWithColors:(NSArray *)colors;

/*!
 * @method gradientWithColorsAndLocations:
 * @abstract Creates a new gradient from a series of colors and locations
 * 
 * @param firstColor
 * A nil-terminated list of colors (as CXColor objects) and locations (as CGFloat values)
 * 
 * @discussion
 * The number of colors and locations must be equal, and they must be in alternating order.
 * 
 * @result A CXGradient object
 */
+ (CXGradient *)gradientWithColorsAndLocations:(CXColor *)firstColor, ... NS_REQUIRES_NIL_TERMINATION;


/*!
 * @property numberOfColorStops
 * @abstract Gets the number of color stops
 * 
 * @result An NSUInteger value
 */
@property (readonly) NSUInteger numberOfColorStops;


/*!
 * @method getColor:location:atIndex:
 * @abtract Retrieves the color and location for the color stop at a specified index
 * 
 * @param color
 * On input, a reference pointer to a CXColor object. On output, the CXColor object at the specified index.
 * 
 * @param location
 * On input, a reference pointer to a CGFloat value. On output, the CGFloat value at the specified index.
 * 
 * @param index
 * The index of the color stop
 */
- (void)getColor:(CXColor **)color location:(CGFloat *)location atIndex:(NSUInteger)index;


/*!
 * @property CGGradient
 * @abstract Gets the CGGradient representation of the gradient
 * 
 * @result A CGGradient object
 */
@property (readonly) CGGradientRef CGGradient;


/*!
 * @method drawInRect:angle:
 * @abstract Draws a linear gradient within a specified rectangle
 * 
 * @param rect
 * The rectangle into which to draw the gradient. The gradient is clipped to this rectangle.
 * 
 * @param angle
 * The angle at which to draw the gradient.
 */
- (void)drawInRect:(CGRect)rect angle:(CGFloat)angle;

/*!
 * @method drawInBezierPath:angle:
 * @abstract Draws a linear gradient within a specified bezier path
 * 
 * @param bezierPath
 * The bezier path into which to draw the gradient. The gradient is clipped to this bezier path.
 * 
 * @param angle
 * The angle at which to draw the gradient.
 */
- (void)drawInBezierPath:(CXBezierPath *)bezierPath angle:(CGFloat)angle;

/*!
 * @method drawFromPoint:toPoint:options:
 * @abstract Draws a linear gradient between two specified points
 * 
 * @param startingPoint
 * The starting point for the gradient. The color at location 0.0 is drawn here.
 * 
 * @param endingPoint
 * The ending point for the gradient. The color at location 1.0 is drawn here.
 * 
 * @param options
 * Options for drawing the gradient
 */
- (void)drawFromPoint:(CGPoint)startingPoint toPoint:(CGPoint)endingPoint options:(CXGradientDrawingOptions)options;

/*!
 * @method drawFromCenter:radius:toCenter:radius:options:
 * @abstract Draws a radial gradient between two center points
 * 
 * @param startCenter
 * The starting center point for the gradient. The color at location 0.0 is drawn here.
 * 
 * @param startRadius
 * The radius of the starting circle
 * 
 * @param endCenter
 * The ending center point for the gradient. The color at location 1.0 is drawn here.
 * 
 * @param endRadius
 * The radius of the ending circle
 * 
 * @param options
 * Options for drawing the gradient
 */
- (void)drawFromCenter:(CGPoint)startCenter radius:(CGFloat)startRadius toCenter:(CGPoint)endCenter radius:(CGFloat)endRadius options:(CXGradientDrawingOptions)options;

@end
