//
//  CXGradient.m
//  Chroniton
//
//  Created by Logan Collins on 5/3/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXGradient.h"
#import "CXColor.h"
#import "CXBezierPath.h"
#import "CXGraphics.h"


@interface CXGradient ()

- (id)initWithColorComponents:(NSArray *)components;

@end


@implementation CXGradient {
    NSArray *_components;
    CGGradientRef _gradient;
}

+ (CXGradient *)gradientWithStartingColor:(CXColor *)startingColor endingColor:(CXColor *)endingColor {
    return [[self alloc] initWithColorComponents:[NSArray arrayWithObjects:
                                                  [NSDictionary dictionaryWithObjectsAndKeys:startingColor, @"color", [NSNumber numberWithDouble:0.0], @"location", nil],
                                                  [NSDictionary dictionaryWithObjectsAndKeys:endingColor, @"color", [NSNumber numberWithDouble:1.0], @"location", nil],
                                                  nil]];
}

+ (CXGradient *)gradientWithColors:(NSArray *)colors {
    if ([colors count] == 0) {
        return nil;
    }
    if ([colors count] == 1) {
        colors = [NSArray arrayWithObjects:[colors lastObject], [colors lastObject], nil];
    }
    
    NSMutableArray *components = [NSMutableArray arrayWithCapacity:[colors count]];
    CGFloat step = (1.0 / (CGFloat)([colors count] - 1));
    for (NSUInteger i=0; i<[colors count]; i++) {
        CXColor *color = [colors objectAtIndex:i];
        CGFloat location = step * (CGFloat)i;
        [components addObject:[NSDictionary dictionaryWithObjectsAndKeys:color, @"color", [NSNumber numberWithDouble:location], @"location", nil]];
    }
    return [[self alloc] initWithColorComponents:components];
}

+ (CXGradient *)gradientWithColorsAndLocations:(CXColor *)firstColor, ... {
    NSMutableArray *components = [NSMutableArray array];
    
    va_list args;
    va_start(args, firstColor);
    for (CXColor *color = firstColor; color != nil; color = va_arg(args, CXColor *)) {
        CGFloat location = va_arg(args, double);
        [components addObject:[NSDictionary dictionaryWithObjectsAndKeys:color, @"color", [NSNumber numberWithDouble:location], @"location", nil]];
    }
    va_end(args);
    
    return [[self alloc] initWithColorComponents:components];
}

- (id)initWithColorComponents:(NSArray *)components {
    self = [super init];
    if (self) {
        _components = [components copy];
    }
    return self;
}

- (id)copyWithZone:(NSZone *)zone {
    CXGradient *gradient = [[[self class] allocWithZone:zone] initWithColorComponents:_components];
    return gradient;
}

- (void)dealloc {
    CGGradientRelease(_gradient);
}

- (CGGradientRef)CGGradient {
    if (_gradient == NULL) {
        CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
        
        CFMutableArrayRef colors = CFArrayCreateMutable(NULL, [_components count], &kCFTypeArrayCallBacks);
        CGFloat * locations = malloc(sizeof(CGFloat) * [_components count]);
        
        for (NSUInteger i=0; i<[_components count]; i++) {
            NSDictionary *colorStop = [_components objectAtIndex:i];
            CGColorRef color = [[colorStop objectForKey:@"color"] CGColor];
            CGFloat location = [[colorStop objectForKey:@"location"] doubleValue];
            CFArrayAppendValue(colors, color);
            locations[i] = location;
        }
        
        _gradient = CGGradientCreateWithColors(colorSpace, colors, locations);
        
        free(locations);
        CFRelease(colors);
        CGColorSpaceRelease(colorSpace);
    }
    return _gradient;
}


#pragma mark -
#pragma mark Components

- (NSUInteger)numberOfColorStops {
    return [_components count];
}

- (void)getColor:(CXColor *__autoreleasing *)color location:(CGFloat *)location atIndex:(NSUInteger)index {
    NSDictionary *colorStop = [_components objectAtIndex:index];
    if (color != NULL) {
        *color = [colorStop objectForKey:@"color"];
    }
    if (location != NULL) {
        *location = [[colorStop objectForKey:@"location"] doubleValue];
    }
}


#pragma mark -
#pragma mark Drawing

- (void)drawInRect:(CGRect)rect angle:(CGFloat)angle {
    CGContextRef c = CXGraphicsGetCurrentContext();
    
    CGPoint startPoint, endPoint;
    
    while (angle < 0.0) {
        angle += 360.0;
    }
    while (angle > 360.0) {
        angle -= 360.0;
    }
    
    if (angle < 90.0) {
        startPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMinY(rect));
        endPoint = CGPointMake(CGRectGetMaxX(rect), CGRectGetMinY(rect));
    }
    else if (angle < 180.0) {
        startPoint = CGPointMake(CGRectGetMaxX(rect), CGRectGetMinY(rect));
        endPoint = CGPointMake(CGRectGetMaxX(rect), CGRectGetMaxY(rect));
    }
    else if (angle < 270.0) {
        startPoint = CGPointMake(CGRectGetMaxX(rect), CGRectGetMaxY(rect));
        endPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMaxY(rect));
    }
    else {
        startPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMaxY(rect));
        endPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMinY(rect));
    }
    
    CGContextSaveGState(c);
    
    CGContextClipToRect(c, rect);
    
    [self drawFromPoint:startPoint toPoint:endPoint options:0];
    
    CGContextRestoreGState(c);
}

- (void)drawInBezierPath:(CXBezierPath *)bezierPath angle:(CGFloat)angle {
    CGContextRef c = CXGraphicsGetCurrentContext();
    CGContextSaveGState(c);
    [bezierPath addClip];
    [self drawInRect:bezierPath.bounds angle:angle];
    CGContextRestoreGState(c);
}

- (void)drawFromPoint:(CGPoint)startingPoint toPoint:(CGPoint)endingPoint options:(CXGradientDrawingOptions)options {
    CGContextRef c = CXGraphicsGetCurrentContext();
    CGGradientRef gradient = [self CGGradient];
    
    CGContextSaveGState(c);
    
    CGGradientDrawingOptions cgOptions = 0;
    if (options & CXGradientDrawsBeforeStartingLocation) {
        cgOptions |= kCGGradientDrawsBeforeStartLocation;
    }
    else if (options & CXGradientDrawsAfterEndingLocation) {
        cgOptions |= kCGGradientDrawsAfterEndLocation;
    }
    
    CGContextDrawLinearGradient(c, gradient, startingPoint, endingPoint, cgOptions);
    
    CGContextRestoreGState(c);
}

- (void)drawFromCenter:(CGPoint)startCenter radius:(CGFloat)startRadius toCenter:(CGPoint)endCenter radius:(CGFloat)endRadius options:(CXGradientDrawingOptions)options {
    CGContextRef c = CXGraphicsGetCurrentContext();
    CGGradientRef gradient = [self CGGradient];
    
    CGContextSaveGState(c);
    
    CGGradientDrawingOptions cgOptions = 0;
    if (options & CXGradientDrawsBeforeStartingLocation) {
        cgOptions |= kCGGradientDrawsBeforeStartLocation;
    }
    else if (options & CXGradientDrawsAfterEndingLocation) {
        cgOptions |= kCGGradientDrawsAfterEndLocation;
    }
    
    CGContextDrawRadialGradient(c, gradient, startCenter, startRadius, endCenter, endRadius, cgOptions);
    
    CGContextRestoreGState(c);
}

@end
