//
//  CXImage.h
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#import <CoreImage/CoreImage.h>
#elif TARGET_OS_MAC
#import <QuartzCore/QuartzCore.h>
#endif


/*!
 * @class CXImage
 * @abstract A high-level representation of an image
 */
@interface CXImage : NSObject

/*!
 * @method imageNamed:
 * @abstract Creates a new image from a file within the bundle
 * 
 * @param name
 * The name of the file within the bundle
 * 
 * @result A CXImage object
 */
+ (CXImage *)imageNamed:(NSString *)name;

/*!
 * @method imageWithContentsOfURL:
 * @abstract Creates a new image from a file URL
 * 
 * @param URL
 * The URL of the file
 * 
 * @result A CXImage object
 */
+ (CXImage *)imageWithContentsOfURL:(NSURL *)URL;

/*!
 * @method imageWithContentsOfFile:
 * @abstract Creates a new image from a file path
 * 
 * @param file
 * The path of the file
 * 
 * @result A CXImage object
 */
+ (CXImage *)imageWithContentsOfFile:(NSString *)file;

/*!
 * @method imageWithCGImage:
 * @abstract Creates a new image from a CGImageRef object
 * 
 * @param cgImage
 * The CGImageRef object
 * 
 * @result A CXImage object
 */
+ (CXImage *)imageWithCGImage:(CGImageRef)cgImage;

/*!
 * @method imageWithCIImage:
 * @abstract Creates a new image from a CIImage object
 * 
 * @param ciImage
 * The CIImage object
 * 
 * @result A CXImage object
 */
+ (CXImage *)imageWithCIImage:(CIImage *)ciImage;


/*!
 * @property size
 * @abstract The size of the image
 * 
 * @result A CGSize value
 */
@property (readonly) CGSize size;

/*!
 * @property CGImage
 * @abstract Returns the CGImageRef object representation
 * 
 * @result A CGImageRef object
 */
@property (readonly) CGImageRef CGImage;

/*!
 * @property CIImage
 * @abstract Returns the CIImage object representation
 * 
 * @result A CIImage object
 */
@property (readonly) CIImage *CIImage;


/*!
 * @method drawInRect:
 * @abstract Draws the image into the current graphics context
 * 
 * @param rect
 * The rect in which to draw
 * 
 * @discussion
 * This method is equivalent to calling drawInRect:blendMode:alpha:
 * with a blend mode of kCGBlendModeNormal and alpha of 1.0.
 */
- (void)drawInRect:(CGRect)rect;

/*!
 * @method drawInRect:blendMode:alpha:
 * @abstract Draws the image into the current graphics context
 * 
 * @param rect
 * The rect in which to draw
 * 
 * @param blendMode
 * The blend mode to use
 * 
 * @param alpha
 * The alpha to use
 */
- (void)drawInRect:(CGRect)rect blendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha;

@end
