//
//  CXControlLayer.m
//  Chroniton
//
//  Created by Logan Collins on 4/26/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXControlLayer.h"
#import "CXControlLayer_Private.h"
#import "CXTouch.h"


static NSString * const CXControlLayerRedisplayPropertyObservationContext = @"CXControlLayerRedisplayPropertyObservationContext";


@interface CXControlTargetAction : NSObject

@property (weak) id target;
@property SEL action;
@property CXControlEvents controlEvents;

@end


@implementation CXControlLayer {
    NSMutableArray *_targetActions;
}

@synthesize state=_state;

- (id)init {
    self = [super init];
    if (self) {
        _targetActions = [NSMutableArray array];
        
        [self addObserver:self forKeyPath:@"state" options:NSKeyValueObservingOptionNew context:(__bridge void *)CXControlLayerRedisplayPropertyObservationContext];
    }
    return self;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if (context == (__bridge void *)CXControlLayerRedisplayPropertyObservationContext) {
        [self setNeedsDisplay];
    }
    else [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
}

- (void)dealloc {
    [self removeObserver:self forKeyPath:@"state"];
}


#pragma mark -
#pragma mark Accessors

+ (NSSet *)keyPathsForValuesAffectingEnabled {
    return [NSSet setWithObjects:@"state", nil];
}

- (BOOL)isEnabled {
    return !(self.state & CXControlStateDisabled);
}

- (void)setEnabled:(BOOL)enabled {
    if (enabled && self.state & CXControlStateDisabled) {
        self.state -= CXControlStateDisabled;
    }
    else if (!enabled && !(self.state & CXControlStateDisabled)) {
        self.state |= CXControlStateDisabled;
    }
}

+ (NSSet *)keyPathsForValuesAffectingSelected {
    return [NSSet setWithObjects:@"state", nil];
}

- (BOOL)isSelected {
    return (self.state & CXControlStateSelected);
}

- (void)setSelected:(BOOL)selected {
    if (selected && !(self.state & CXControlStateSelected)) {
        self.state |= CXControlStateSelected;
    }
    else if (!selected && (self.state & CXControlStateSelected)) {
        self.state -= CXControlStateSelected;
    }
}

+ (NSSet *)keyPathsForValuesAffectingHighlighted {
    return [NSSet setWithObjects:@"state", nil];
}

- (BOOL)isHighlighted {
    return (self.state & CXControlStateHighlighted);
}

- (void)setHighlighted:(BOOL)highlighted {
    if (highlighted && !(self.state & CXControlStateHighlighted)) {
        self.state |= CXControlStateHighlighted;
    }
    else if (!highlighted && (self.state & CXControlStateHighlighted)) {
        self.state -= CXControlStateHighlighted;
    }
}


#pragma mark -
#pragma mark Actions

- (void)addTarget:(id)target action:(SEL)action forControlEvents:(CXControlEvents)controlEvents {
    if (target == nil) {
        target = [NSNull null];
    }
    if (action == nil) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"action cannot be nil" userInfo:nil];
    }
    
    CXControlTargetAction *targetAction = [[CXControlTargetAction alloc] init];
    targetAction.target = target;
    targetAction.action = action;
    targetAction.controlEvents = controlEvents;
    
    [_targetActions addObject:targetAction];
}

- (void)removeTarget:(id)target action:(SEL)action forControlEvents:(CXControlEvents)controlEvents {
    NSArray *targetActions = [_targetActions copy];
    for (CXControlTargetAction *targetAction in targetActions) {
        if (targetAction.target == target
            && (action == nil || action == targetAction.action)) {
            [_targetActions removeObject:targetAction];
        }
    }
}

- (NSArray *)actionsForTarget:(id)target forControlEvents:(CXControlEvents)controlEvents {
    NSMutableArray *actions = [NSMutableArray array];
    for (CXControlTargetAction *targetAction in _targetActions) {
        if (targetAction.target == target
            && (targetAction.controlEvents & controlEvents)) {
            [actions addObject:NSStringFromSelector(targetAction.action)];
        }
    }
    return [actions copy];
}

- (NSArray *)allTargets {
    NSMutableSet *targets = [NSMutableSet set];
    for (CXControlTargetAction *targetAction in _targetActions) {
        [targets addObject:targetAction.target];
    }
    return [targets allObjects];
}

- (CXControlEvents)allControlEvents {
    CXControlEvents controlEvents = 0;
    for (CXControlTargetAction *targetAction in _targetActions) {
        controlEvents |= targetAction.controlEvents;
    }
    return controlEvents;
}

- (NSArray *)targetActionsMatchingControlEvents:(CXControlEvents)controlEvents {
    return [_targetActions filteredArrayUsingPredicate:[NSPredicate predicateWithBlock:^BOOL(id evaluatedObject, NSDictionary *bindings) {
        return ([evaluatedObject controlEvents] & controlEvents);
    }]];
}

- (void)sendActionsForControlEvents:(CXControlEvents)controlEvents forEvent:(CXEvent *)event {
    NSArray *targetActions = [self targetActionsMatchingControlEvents:controlEvents];
    for (CXControlTargetAction *targetAction in targetActions) {
        [self sendAction:targetAction.action to:targetAction.target forEvent:event];
    }
}

- (void)sendAction:(SEL)action to:(id)target forEvent:(CXEvent *)event {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    [target performSelector:action withObject:self withObject:event];
#pragma clang diagnostic pop
}


#pragma mark -
#pragma mark Responder

- (void)touchesBegan:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    self.highlighted = YES;
    [self sendActionsForControlEvents:CXControlEventTouchDown forEvent:theEvent];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    CXTouch *touch = [touches anyObject];
    CGPoint location = [touch locationInLayer:self];
    CGPoint previousLocation = [touch previousLocationInLayer:self];
    if (CGRectContainsPoint(self.bounds, location)) {
        // In bounds
        if (CGRectContainsPoint(self.bounds, previousLocation)) {
            [self sendActionsForControlEvents:CXControlEventTouchDragInside forEvent:theEvent];
        }
        else {
            self.highlighted = YES;
            [self sendActionsForControlEvents:CXControlEventTouchDragEnter forEvent:theEvent];
        }
    }
    else {
        // Out of bounds
        if (CGRectContainsPoint(self.bounds, previousLocation)) {
            [self sendActionsForControlEvents:CXControlEventTouchDragOutside forEvent:theEvent];
        }
        else {
            self.highlighted = NO;
            [self sendActionsForControlEvents:CXControlEventTouchDragExit forEvent:theEvent];
        }
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    self.highlighted = NO;
    CXTouch *touch = [touches anyObject];
    CGPoint location = [touch locationInLayer:self];
    if (CGRectContainsPoint(self.bounds, location)) {
        // In bounds
        [self sendActionsForControlEvents:CXControlEventTouchUpInside forEvent:theEvent];
    }
    else {
        // Out of bounds
        [self sendActionsForControlEvents:CXControlEventTouchUpOutside forEvent:theEvent];
    }
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(CXEvent *)theEvent {
    self.highlighted = NO;
    [self sendActionsForControlEvents:CXControlEventTouchCancel forEvent:theEvent];
}

@end


@implementation CXControlTargetAction

@synthesize target=_target;
@synthesize action=_action;
@synthesize controlEvents=_controlEvents;

@end
