//
//  CXDirector.m
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXDirector.h"
#import "CXTimer.h"
#import "CXScene.h"


NSString * const CXDirectorDidPauseNotification = @"CXDirectorDidPauseNotification";
NSString * const CXDirectorDidResumeNotification = @"CXDirectorDidResumeNotification";


@interface CXDirector ()

- (void)invalidateAllTimers;

@end


@implementation CXDirector {
	NSMutableArray *_timers;
	NSMutableDictionary *_runLoopTimersToTimers;
	NSMutableOrderedSet *_scenes;
}

@synthesize delegate=_delegate;

@synthesize baseLayer=_baseLayer;
@synthesize paused=_paused;

+ (CXDirector *)sharedDirector {
	static CXDirector *sharedDirector = nil;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedDirector = [[self alloc] init];
	});
	return sharedDirector;
}

- (id)init {
	self = [super init];
	if (self) {
		_baseLayer = [CALayer layer];
		
		_timers = [NSMutableArray array];
		_runLoopTimersToTimers = [NSMutableDictionary dictionary];
		
		_scenes = [NSMutableOrderedSet orderedSet];
	}
	return self;
}

- (void)dealloc {
	[self invalidateAllTimers];
}


#pragma mark -
#pragma mark Timing

- (void)pause {
	if (_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	
	_paused = YES;
	
	// Deschedule timers
	for (CXTimer *timer in _timers) {
		NSTimer *runLoopTimer = [timer runLoopTimer];
		NSDate *fireDate = [runLoopTimer fireDate];
		
		NSDate *date = [NSDate date];
		NSTimeInterval remainingTimeInterval = [fireDate timeIntervalSinceDate:date];
		
		timer.nextInterval = remainingTimeInterval;
		
		[runLoopTimer invalidate];
		[_runLoopTimersToTimers removeObjectForKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	}
	
	[self didChangeValueForKey:@"paused"];
	
	if ([[self delegate] respondsToSelector:@selector(directorDidPause:)]) {
		[[self delegate] directorDidPause:self];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:CXDirectorDidPauseNotification object:self];
}

- (void)resume {
	if (!_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	
	_paused = NO;
	
	// Reschedule timers
	for (CXTimer *timer in _timers) {
		NSTimeInterval nextInterval = [timer nextInterval];
		NSTimer *runLoopTimer = [NSTimer scheduledTimerWithTimeInterval:nextInterval target:self selector:@selector(executeTimer:) userInfo:nil repeats:NO];
		
		timer.runLoopTimer = runLoopTimer;
		
		[_runLoopTimersToTimers setObject:timer forKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	}
	
	[self didChangeValueForKey:@"paused"];
	
	if ([[self delegate] respondsToSelector:@selector(directorDidResume:)]) {
		[[self delegate] directorDidResume:self];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:CXDirectorDidResumeNotification object:self];
}

- (void)executeTimer:(NSTimer *)runLoopTimer {
	CXTimer *timer = [_runLoopTimersToTimers objectForKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	void (^block)(void) = timer.block;
	if (block != nil) {
		block();
	}
	
	// Remove the timer if it doesn't repeat
	[_runLoopTimersToTimers removeObjectForKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	if (!timer.repeats) {
		[_timers removeObject:timer];
	}
	else {
		NSTimer *runLoopTimer = [NSTimer scheduledTimerWithTimeInterval:timer.interval target:self selector:@selector(executeTimer:) userInfo:nil repeats:NO];
		timer.runLoopTimer = runLoopTimer;
		[_runLoopTimersToTimers setObject:timer forKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	}
}

- (id)scheduleTimerWithInterval:(NSTimeInterval)interval repeats:(BOOL)repeats block:(void (^)(void))block {
	if (block == nil) {
		return nil;
	}
	
	CXTimer *timer = [CXTimer timerWithInterval:interval repeats:repeats block:block];
	[_timers addObject:timer];
	
	if (!_paused) {
		NSTimer *runLoopTimer = [NSTimer scheduledTimerWithTimeInterval:interval target:self selector:@selector(executeTimer:) userInfo:nil repeats:NO];
		timer.runLoopTimer = runLoopTimer;
		[_runLoopTimersToTimers setObject:timer forKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	}
	else {
		timer.nextInterval = interval;
	}
	
	return timer;
}

- (void)invalidateTimer:(id)timer {
	if (timer == nil) {
		return;
	}
	
	[_timers removeObject:timer];
	
	NSTimer *runLoopTimer = [timer runLoopTimer];
	if (runLoopTimer != nil) {
		[runLoopTimer invalidate];
		[_runLoopTimersToTimers removeObjectForKey:[NSNumber numberWithInteger:[runLoopTimer hash]]];
	}
}

- (void)invalidateAllTimers {
	for (CXTimer *timer in _timers) {
		[self invalidateTimer:timer];
	}
}


#pragma mark -
#pragma mark Scenes

- (NSArray *)scenes {
	return [[_scenes array] copy];
}

- (CXScene *)currentScene {
	return [_scenes lastObject];
}

- (void)setCurrentScene:(CXScene *)currentScene {
	if ([_scenes containsObject:currentScene]) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene is already in the scene stack." userInfo:nil];
	}
	if (currentScene == nil) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene cannot be nil." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *previousScene = [_scenes lastObject];
	if (previousScene != nil) {
		[previousScene sceneWillEnd];
		[[previousScene layer] removeFromSuperlayer];
		[_scenes removeObject:previousScene];
		[previousScene sceneDidEnd];
	}
	
	[currentScene sceneWillStart];
	
	[_scenes addObject:currentScene];
	
	CALayer *sceneLayer = [currentScene layer];
	sceneLayer.autoresizingMask = (kCALayerWidthSizable|kCALayerHeightSizable);
	sceneLayer.frame = self.baseLayer.bounds;
	[self.baseLayer addSublayer:sceneLayer];
	
	[currentScene sceneDidStart];
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
}

- (void)pushScene:(CXScene *)scene {
	if ([_scenes containsObject:scene]) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene is already in the scene stack." userInfo:nil];
	}
	if (scene == nil) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Scene cannot be nil." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *currentScene = [self currentScene];
	if (currentScene != nil) {
		[currentScene sceneWillPause];
		[[currentScene layer] removeFromSuperlayer];
		[currentScene sceneDidPause];
	}
	
	[scene sceneWillStart];
	
	[_scenes addObject:scene];
	
	CALayer *sceneLayer = [scene layer];
	sceneLayer.autoresizingMask = (kCALayerWidthSizable|kCALayerHeightSizable);
	sceneLayer.frame = self.baseLayer.bounds;
	[self.baseLayer addSublayer:sceneLayer];
	
	[scene sceneDidStart];
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeInsertion valuesAtIndexes:indexes forKey:@"scenes"];
}

- (void)popScene {
	if ([_scenes count] == 0) {
		@throw [NSException exceptionWithName:NSRangeException reason:@"There are no scenes in the scene stack to pop." userInfo:nil];
	}
	
	NSIndexSet *indexes = [NSIndexSet indexSetWithIndex:[_scenes count]];
	[self willChange:NSKeyValueChangeRemoval valuesAtIndexes:indexes forKey:@"scenes"];
	[self willChangeValueForKey:@"currentScene"];
	
	CXScene *currentScene = [self currentScene];
	[currentScene sceneWillEnd];
	[[currentScene layer] removeFromSuperlayer];
	[_scenes removeObject:currentScene];
	[currentScene sceneDidEnd];
	
	CXScene *nextScene = [_scenes lastObject];
	if (nextScene != nil) {
		[nextScene sceneWillResume];
		
		CALayer *sceneLayer = [nextScene layer];
		sceneLayer.autoresizingMask = (kCALayerWidthSizable|kCALayerHeightSizable);
		sceneLayer.frame = self.baseLayer.bounds;
		[self.baseLayer addSublayer:sceneLayer];
		
		[nextScene sceneDidResume];
	}
	
	[self didChangeValueForKey:@"currentScene"];
	[self didChange:NSKeyValueChangeRemoval valuesAtIndexes:indexes forKey:@"scenes"];
}

@end
