//
//  CXScene.h
//  Chroniton
//
//  Created by Logan Collins on 4/3/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Chroniton/ChronitonDefines.h>
#import <Chroniton/CXResponder.h>
#import <QuartzCore/QuartzCore.h>


@class CXDirector, CXEvent, CXLayer;


/*!
 * @class CXScene
 * @abstract Coordinates scene actions (animation, sound, timing, etc.)
 */
@interface CXScene : NSObject <CXResponder>

/*!
 * @method scene
 * @abstract Creates a new scene
 * 
 * @result A CXScene object
 */
+ (id)scene;


/*!
 * @property layer
 * @abstract The scene's base layer
 * 
 * @result A CXLayer object
 */
@property (strong, readonly) CXLayer *layer;


/*!
 * @property director
 * @abstract The director in control of the scene
 * 
 * @result A CXDirector object
 */
@property (weak, readonly) CXDirector *director;


/*!
 * @method sceneWillStart
 * @abstract Notifies that the scene is about to start
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneWillStart;

/*!
 * @method sceneDidStart
 * @abstract Notifies that the scene has started
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneDidStart;

/*!
 * @method sceneWillPause
 * @abstract Notifies that the scene is about to pause
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneWillPause;

/*!
 * @method sceneDidPause
 * @abstract Notifies that the scene has paused
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneDidPause;

/*!
 * @method sceneWillResume
 * @abstract Notifies that the scene is about to resume
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneWillResume;

/*!
 * @method sceneDidResume
 * @abstract Notifies that the scene has resumed
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneDidResume;

/*!
 * @method sceneWillEnd
 * @abstract Notifies that the scene is about to end
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneWillEnd;

/*!
 * @method sceneDidEnd
 * @abstract Notifies that the scene has ended
 * 
 * @discussion
 * Subclasses should call super somewhere in their implementation.
 */
- (void)sceneDidEnd;


/*!
 * @method convertPointFromBase:
 * @abstract Converts a point from the base geometry of the viewport to the scene's layer
 * 
 * @param basePoint
 * The point to convert, in screen viewport coordinates
 * 
 * @result A CGPoint value
 */
- (CGPoint)convertPointFromBase:(CGPoint)basePoint;

/*!
 * @method convertPointToBase:
 * @abstract Converts a point from the scene's layer to the base geometry of the viewport
 * 
 * @param scenePoint
 * The point to convert, in scene layer coordinates
 * 
 * @result A CGPoint value
 */
- (CGPoint)convertPointToBase:(CGPoint)scenePoint;

/*!
 * @method convertRectFromBase:
 * @abstract Converts a rect from the base geometry of the viewport to the scene's layer
 * 
 * @param baseRect
 * The rect to convert, in screen viewport coordinates
 * 
 * @result A CGRect value
 */
- (CGRect)convertRectFromBase:(CGRect)baseRect;

/*!
 * @method convertRectToBase:
 * @abstract Converts a rect from the scene's layer to the base geometry of the viewport
 * 
 * @param sceneRect
 * The rect to convert, in scene layer coordinates
 * 
 * @result A CGRect value
 */
- (CGRect)convertRectToBase:(CGRect)sceneRect;

@end
